import {EntityRepository, getConnection, getManager, Repository} from "typeorm";
import {CreateBlogPostDto} from "./dto/create-blog-post.dto";
import {UpdateBlogPostDto} from "./dto/update-blog-post.dto";
import {BlogPost} from "./entities/blog-post.entity";

@EntityRepository(BlogPost)
export class BlogPostRepository extends Repository<BlogPost> {

    async createEntity(dto: CreateBlogPostDto): Promise<any | Error> {
        try {
            const entity = this.create({
                categoryId: dto.categoryId,
                categoryName: dto.categoryName,
                title: dto.title,
                photo: dto.photo,
                longDetails: dto.longDetails,
                urlSlug: dto.urlSlug
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                order: {
                    createdAt: -1
                },
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    id: id
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntityBySlug(urlSlug: string): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    urlSlug: urlSlug
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(BlogPost)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async updateBlogPost(id: string, dto: UpdateBlogPostDto): Promise<BlogPost | Error> {
        try {
            await this.update(id, {
                categoryId: dto.categoryId,
                categoryName: dto.categoryName,
                title: dto.title,
                photo: dto.photo,
                longDetails: dto.longDetails,
                urlSlug: dto.urlSlug
            });
            return await this.findOne({where: {id: id}});
        } catch (e) {
            return e;
        }
    }

    async updateBlogViewCount(id: string, dto: UpdateBlogPostDto): Promise<BlogPost | Error> {
        try {
            await this.update(id, {
                totalViews: dto.totalViews
            });
            return await this.findOne({where: {id: id}});
        } catch (e) {
            return e;
        }
    }

    async getTopBlogList(): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    totalViews: "DESC"
                },
                take: 3
            });
        } catch (e) {
            return e;
        }
    }

    async getLatestBlogList(): Promise<BlogPost[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                },
                take: 3
            });
        } catch (e) {
            return e;
        }
    }

    async countAllBlogs(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findBlogByFilterQuery(
        page: number,
        limit: number,
        query: any
    ): Promise<any | Error> {
        try {
            const count = await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
            const posts = await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                take: limit,
                skip: limit * (page - 1)
            });
            const pageCount = Math.ceil(count.length / limit);
            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                posts
            };
        } catch (e) {
            return e;
        }
    }

    async findBlogByTagName(page: number, limit: number, tagName: any): Promise<any | Error> {
        try {
            let sql = `SELECT blog_posts.* FROM blog_posts 
                       LEFT JOIN blog_post_tags 
                       ON blog_posts.id = blog_post_tags.postId 
                       WHERE blog_post_tags.tagName = '${tagName}' AND blog_post_tags.isDeleted = 0;`;
            const entityManager = getManager();
            const posts = await entityManager.query(sql);
            const count = posts.length;
            const pageCount = Math.ceil(count / limit);
            const responseData = {
                page: page,
                limit: limit,
                pageCount: pageCount,
                totalData: count,
                posts
            };
            return responseData;
        } catch (e) {
            return e;
        }
    }
}